package nckd.jimin.jyyy.bd.plugin.msg.ecology;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.workflow.exception.WFErrorCode;
import kd.sdk.plugin.Plugin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class EcologyOpPlugin extends AbstractOperationServicePlugIn implements Plugin {
    private static Log logger = LogFactory.getLog(EcologyOpPlugin.class);
    private static final String KEY_ENTITY_LOG = "nckd_ecology";
    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);

        StringBuffer errMsg = new StringBuffer();

        //获取选中行
        DynamicObject[] dynamicObjects = e.getDataEntities();

        //获取操作按钮操作编码
        String operationKey = e.getOperationKey();
        if (StringUtils.equals( "sendmsg",operationKey)){
            List<DynamicObject> listObj = new ArrayList<>();
            String selectField = "nckd_ecologyurl, nckd_ecologycode, nckd_ecologmsgcode, nckd_ecologappid, nckd_ecologsecret, nckd_ecologspk, nckd_ecologyurlmsg";
            DynamicObject[] commonParams = BusinessDataServiceHelper.load("nckd_commonparams", selectField, null);
            String FANWEI_URL = "";
            if(commonParams.length > 0){
                FANWEI_URL = commonParams[0].getString("nckd_ecologyurl");
            } else {
                return;
            }

            for(int i = 0; i < dynamicObjects.length; i++) {
                String id = dynamicObjects[i].getPkValue().toString();
                DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(id, KEY_ENTITY_LOG);
                Boolean needSend = false;
                if(dynamicObject.getString("nckd_logtype").matches("1|5")
                    && "fail".equals(dynamicObject.getString("nckd_sendstatus"))){
                    needSend = true;
                }

                if(!needSend){
                    continue;
                }

                JSONObject fanweiBodyData = JSONObject.parseObject(dynamicObject.getString("nckd_senddata_tag"));
//                FANWEI_URL = dynamicObject.getString("nckd_sendurl");
                String url = FANWEI_URL + "/rest/ofs/ReceiveRequestInfoByJson";
                JSONObject result = doPostByHttpClient(url, fanweiBodyData);
                if (result == null) {
                    throw new KDException(WFErrorCode.httpRequestWrongResponse(), new Object[]{result != null ? result.toJSONString() : "result is null!"});
                }

                String sendStatus = "fail";
                //失败
                if ("1".equals(result.getString("operResult"))) {
                    sendStatus = "success";
                    logger.info("EcologyOpPlugin：待办(" + dynamicObject.getString("number") + ")重发成功!");
                    errMsg.append(dynamicObject.getString("number")).append(":").append("重发成功!").append("\r\n");
                } else {
                    logger.info("EcologyOpPlugin：待办(" + dynamicObject.getString("number") + ")重发失败" + result.getString("message"));
                    errMsg.append(dynamicObject.getString("number")).append(":").append("重发失败!").append("\r\n");
                }

                String response = result.toJSONString();
                if (response.length() < 200) {
                    dynamicObject.set("nckd_resultdata", response);
                } else {
                    dynamicObject.set("nckd_resultdata", response.substring(0, 200) + "...");
                }
                dynamicObject.set("nckd_resultdata_tag", response);
                dynamicObject.set("nckd_sendstatus", sendStatus);
                dynamicObject.set("nckd_sendurl", FANWEI_URL);

                listObj.add(dynamicObject);

                if(listObj.size() == 500) {
                    SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
                    listObj.clear();
                }
            }
            if(listObj.size() > 0) {
                SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
            }
        }

        if(errMsg != null && errMsg.length() > 0) {
            ////将错误信息返回到前端
            e.setCancelMessage(errMsg.toString());
            e.setCancel(true);
            System.out.println("EcologyOpPlugin 错误信息：" + errMsg.toString());
        }
    }

    public static JSONObject doPostByHttpClient(String url, JSONObject bodyData) {
        try {
            Map<String, String> headers = new HashMap();
            headers.put("Content-Type", "application/json; charset=utf-8");
            headers.put("Accept", "application/json");
            logger.info(String.format("url[%s],data[%s]", url, bodyData.toJSONString()));
            System.out.println(String.format("url[%s],data[%s]", url, bodyData.toJSONString()));

            String responseEntify = HttpUtils.postjson(url, headers, bodyData.toJSONString());
            JSONObject result = (JSONObject)JSONObject.parse(responseEntify);
            return result;
        } catch (IOException var5) {
            throw new KDException(var5, WFErrorCode.httpRequestException(), new Object[]{var5.getMessage()});
        }
    }
}
